/* h264.h
 * simple h264 stream parser
 * Copyright(c) 2012 Sensoray Co., Inc.
 */

#ifndef _H264_H_
#define _H264_H_

#ifdef __cplusplus
extern "C" {
#endif

extern int h264_info(unsigned char *buf, int len, int *profile, int *level, int *width, int *height,
					 int *num_units_in_tick, int *time_scale);

#ifdef __cplusplus
}
#endif

#endif
